"""WARNING: Autogenerated by django-settings-model."""

DEBUG = True
SECRET_KEY = '2#mu)bp#5-2bru4n#yijvbp$isc=@9yvrtk_$cz2uvi7ziwd20'
TIME_ZONE = 'America/Chicago'
APPEND_SLASH = True
ALLOWED_HOSTS = ['*']
