import os
from setuptools import find_packages, setup

import settings_model


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# get README
with open("README.rst") as f:
    long_description = f.read()

setup(
    name="django-settings-model",
    version=settings_model.__version__,
    packages=find_packages(),
    install_requires=["Django>=2", "pytz"],
    description=(
        "A re-useable Django app for building models that modify Django settings."
    ),
    long_description=long_description,
    url="https://github.com/gregschmit/django-settings-model",
    author="Gregory N. Schmit",
    author_email="me@gregschmit.com",
    license="MIT",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)
